/*
    Program pro astronomicke hodiny

    Detaily + popis hardwaru:
      http://www.astromik.org/raspi/astrohodiny/index.htm

    prvni zverejnena verze

    Datum: 17.11.2014

*/



#include <EEPROM.h>
#include <Wire.h>
#include <rtc_m41t81.h>
#include <nokia_b.h>


#include <SPI.h>      // seriova komunikace s pameti
#define ramCS_pin 8   // natvrdo nastaveny CS pin od RAM pameti na vyvod D8



// vytvori objekt nokia_b s nazvem lcd na pinech 7,6,5,4,3,9
//          rst, ce, dc, in, clk, podsv)
nokia_b lcd( 7 , 6 , 5 , 4 ,  3 ,   9  ); // jako original, ale se samostatnym podsvetem na pinu D9




// zakladni nastaveni RTC (hlavne jde o 1Hz vystup)
rtc_m41t81 rtc;




char* cas;

byte alarm_cas[5];        // 5 bajtu s mesicem,dnem,hodinou,minutou a sekundou ve  formatu primo pouzitelnem pro RTC obvod (registry 0x0A az 0x0E)
byte alarm_spec;          // 3 bajty se specialnimi informacemi (rezervy)
char alarm_koment[57];    // 56-znakovy komentar ke kazdemu alarmu

byte napetimax1;
byte napetimax2;

word adresaFAT;
word pocet_zaznamu_FATbloku;

byte bzuk_flag[4];
byte thus_flag[3];

word perioda_save;
word cyklus_periody_save;

short cas_param[6];

// globalni promenne pro jednotlive cifry datumu a casu z RTC
char sekjed, sekdes, minjed, mindes, hodjed, hoddes, denjed, dendes, mesjed, mesdes, rokjed, rokdes;

// globalni promenne a aktalnim datumem a casem v RTC prevedenym do citelne soustavy
byte hoddec, mindec, sekdec, mscdec, dendec, mesdec, rokdec ;

int indexmenu;
int delpauza;

byte ala_count;   // celkovy pocet aktivnich alarmu
byte ala_max;     // celkovy pocet vsech alarmu (i neaktivnich i uzivatelskych - takze minimalne 5)
word zaz_count;   // celkovy pocet zaznamenanych casovych okamziku
byte mem_proc;    // aktualni stav volne pameti SRAM pro zaznam teploty ....

boolean inverzem; // znacka pro inverzni zobrazeni pismene "M" v inforamecku pri ukladani namerenych hodnot

word zaz_index;   // prubezny index zaznamu (cislo nasledujiciho zaznamu)
boolean ukazzaznam;

boolean rotace_podtextu;
byte rotace_citac;

byte dalsi_alarm;
byte predchozi_alarm;

int kurzor_x;
int kurzor_y;
byte x_edit;

byte i;
byte smer_j;    // promenna pro ulozeni smeru vychyleni joysticku pri zaznamu casoveho okamziku

unsigned long durat_start;  // doba vychyleni joysticku pri zaznamu casoveho okamziku
byte durat_j;

word  adresa;     // pouzito pro adresovani externi pameti SRAM, nebo vnitrni EEPROM

volatile boolean inter_flag;  // informace o tom, ze doslo k interruptu od RTC obvodu (kazdou sekundu)

short joystick_x;
short joystick_y;

boolean uvolneno=false ;

volatile int jas;

byte index_dalsiho_alarmu;

unsigned long cas_dalsiho_alarmu = 0xFFFFFFFFUL;
unsigned long aktcas;
unsigned long longalarm;

unsigned long minipauza;

byte kalibrace;

byte indexpodtextu = 0;


// =====================  podprogramy pro praci s RAM ====================
void ram_disable()
{
  digitalWrite(ramCS_pin, HIGH);
}

void ram_enable()
{
  digitalWrite(ramCS_pin, LOW);
}


void zapis_bajt(word adresa, byte data)
{
  ram_enable();                    // CS do "0"
  SPI.transfer(0x02);              // nastaveni instrukce pro zapis
  SPI.transfer(adresa >> 8);       // posle horni polovinu adresy
  SPI.transfer(adresa & 0x00FF);   // posle spodni polovinu adresy
  SPI.transfer(data);              // posle datovy bajt
  ram_disable();                   // CS do "1"
}


byte precti_bajt(word adresa)
{
  ram_enable();                    // CS do "0"
  SPI.transfer(0x03);              // nastaveni instrukce pro cteni
  SPI.transfer(adresa >> 8);       // posle horni polovinu adresy
  SPI.transfer(adresa & 0x00FF);   // posle spodni polovinu adresy
  byte data = SPI.transfer(0x00);  // posle libovolny bajt jen kvuli tikani hodin (nezapisuje se)
  ram_disable();                   // CS do "1"
  return data;
}

void ram_init()
{
  pinMode(ramCS_pin,   OUTPUT);   //CS pin na vystup
  SPI.begin();                    // start SPI komunikace
  ram_disable();                  // na CS pinu nastavit 1 (nekomunikovat)

  // nastaveni parametru SPI komunikace
  SPI.setClockDivider(0);       // komunikacni rychlost na max (4MHz) - pamet zvlada az 20MHz
  SPI.setBitOrder(MSBFIRST);    // prvni prenaseny bit je nejvyssi
  SPI.setDataMode(SPI_MODE0);   // hodiny jsou iddle v "1"  a pepisovaci hrana je nabezna (L->H)

  // nastaveni WRMR registru
  ram_enable();
  SPI.transfer(0x01);   // adresa WRMR registru
  SPI.transfer(0x00);   // rezim prenosu jednotlivych bajtu
  ram_disable();
}
// =====================  konec podprogramu pro praci s RAM ====================



void delay500()
{
  delay(500);
}

void delay200()
{
  delay(200);
}



void delay100()
{
  delay(100);
}


void kod46()
{
  lcd.LcdKod(46);
}

void kod32()
{
  lcd.LcdKod(32);
}

void kod58()
{
  lcd.LcdKod(58);
}

void kod0x80()
{
  lcd.LcdKod(0x80);
}

void piip()
{
  tone(A2, 2000, 40);
}

void jmp0()
{
 asm volatile ("jmp 0");
}


void lcdgoto00()
{
  lcd.GotoXY(0, 0);
}

void lcdgoto01()
{
  lcd.GotoXY(0, 1);
}


void lcdgoto02()
{
  lcd.GotoXY(0, 2);
}

void lcdgoto04()
{
  lcd.GotoXY(0, 4);
}

void lcdgoto05()
{
  lcd.GotoXY(0, 5);
}





//prevadi aktualni cas v RTC do globalnic promennych rokdec, mesdec, dendec ....mscdec
void rtc_do_datcas(void)
{
  cas = rtc.formatTime(1);
  hoddec=((cas[0]-'0') * 10) + (cas[1]-'0');
  mindec=((cas[2]-'0') * 10) + (cas[3]-'0');
  sekdec=((cas[4]-'0') * 10) + (cas[5]-'0');
  mscdec=((cas[6]-'0') * 10) + (cas[7]-'0');

  dendec=((cas[8]-'0') * 10) + (cas[9]-'0');
  mesdec=((cas[10]-'0') * 10) + (cas[11]-'0');
  rokdec=((cas[12]-'0') * 10) + (cas[13]-'0');


}



void novyFATzaznam(void)
{
  byte cisloFAT;

  if (precti_bajt(11) > 0)         // obsah THUS flagu
    {
      inverzem = true;
      cisloFAT = precti_bajt(13);
      if (cisloFAT < 201)
        {
          rtc_do_datcas();
          adresaFAT = 15484 + (cisloFAT * 16);

          zapis_bajt(adresaFAT,cisloFAT);   // index FAT bloku
          zapis_bajt(adresaFAT+1,rokdec);   // start mereni jednoho FAT bloku
          zapis_bajt(adresaFAT+2,mesdec);
          zapis_bajt(adresaFAT+3,dendec);
          zapis_bajt(adresaFAT+4,hoddec);
          zapis_bajt(adresaFAT+5,mindec);
          zapis_bajt(adresaFAT+6,sekdec);

          zapis_bajt(adresaFAT+7,precti_bajt(9));  // perioda
          zapis_bajt(adresaFAT+8,precti_bajt(10));

          zapis_bajt(adresaFAT+9,precti_bajt(4));  // startovni adresa bloku namerenych hodnot
          zapis_bajt(adresaFAT+10,precti_bajt(5));

          zapis_bajt(adresaFAT+11,precti_bajt(11));  // THUS flagy

          zapis_bajt(adresaFAT+12,0);  // pocet zaznamu v jednom FAT bloku - na zacatku pri zakladani FAT je to 0
          zapis_bajt(adresaFAT+13,0);

          zapis_bajt(adresaFAT+14,precti_bajt(18));  // ulozeni napetovych rozsahu do FAT
          zapis_bajt(adresaFAT+15,precti_bajt(19));

          pocet_zaznamu_FATbloku = 0;

          cisloFAT++;
          zapis_bajt(13,cisloFAT);
        }
      else
        {
          mem_proc = 233;
          zapis_bajt(13,201);  // prekroceni maximalniho poctu bloku FAT
          lcd.ramecek(ala_count,zaz_count,mem_proc,true);
        }
    }
  else
    {
      inverzem = false;
    }

}

void uloz_mereni(void)
{
  word napeti;

  if (precti_bajt(11) > 0)
    {
      adresa = (precti_bajt(4) * 256) + precti_bajt(5) ; // adresa volneho mista, kam se budou ukladat namerene hodnoty

      if (thus_flag[0]==1)   // teplota
        {
          Wire.beginTransmission(0x4B);
          Wire.write(0x00);

          Wire.requestFrom(0x4B, 2);
          zapis_bajt(adresa, Wire.read());
          zapis_bajt(adresa + 1, Wire.read());
          adresa = adresa  + 2;
        }

      if (thus_flag[1]==1)   // napeti1
        {
          napeti = analogRead(A3);
          zapis_bajt(adresa, napeti/256);
          zapis_bajt(adresa + 1,napeti%256);
          adresa = adresa + 2;
        }

      if (thus_flag[2]==1)   // napeti2
        {
          napeti = analogRead(A7);
          zapis_bajt(adresa, napeti/256);
          zapis_bajt(adresa + 1,napeti%256);
          adresa = adresa + 2;
        }


      if ((adresa / 256) == 255 && (adresa % 256) > 240  )
        {
          mem_proc = 211; // prekroceni maximalniho prostoru pro mereni
        }
      else
        {
          if (mem_proc<200)
            {

              zapis_bajt(4,adresa / 256);
              zapis_bajt(5,adresa % 256);
              pocet_zaznamu_FATbloku ++;
              zapis_bajt(adresaFAT+12,pocet_zaznamu_FATbloku >> 8);
              zapis_bajt(adresaFAT+13,pocet_zaznamu_FATbloku & 0xFF);

              mem_proc  = (((-1*(precti_bajt(4)))+255)/1.83);   // procenta volneho mista


              if (bzuk_flag[3]==1)
                {
                  piip();
                }
            }
        }
    }
}




void preruseni()   // hlavni preruseni je generovano RTC obvodem kazdou sekundu
{

  inter_flag = true;
}

unsigned int read16x(void)
{
  int val;

  Wire.beginTransmission(0x4B);
  Wire.write(0x00);

  Wire.requestFrom(0x4B, 2);
  val = Wire.read();
  val <<= 8;
  val |= Wire.read();
  return val;
}


unsigned int teplota(void)
{
  detachInterrupt(0);
  unsigned int temp = read16x();
  attachInterrupt(0, preruseni, FALLING);
  return temp;
}





void zobrazcas()
{
  unsigned int unap;
  unsigned int tep;
  float tepcitelna;
  unsigned int batnap;
  byte batproc;
  float vypoctene_cislo;
  byte tepkalib;


  cas = rtc.formatTime(2);
  hoddes = cas[0];
  if (hoddes == '0')
    {
      hoddes = ' ';
    }
  hodjed = cas[1];
  mindes = cas[3];
  minjed = cas[4];
  sekdes = cas[6];
  sekjed = cas[7];

  sekdec=((cas[6]-'0') * 10) + (cas[7]-'0');

  mesdec=((cas[9]-'0') * 10) + (cas[10]-'0');
  dendec=((cas[11]-'0') * 10) + (cas[12]-'0');



  if ((mindes == '0') && (minjed=='0')  && (sekdec == 0)  && bzuk_flag[0]==1) tone(A2, 700, 60);  // pipani v celou hodinu


  cyklus_periody_save ++;
  if (cyklus_periody_save > perioda_save-1)
    {
      cyklus_periody_save =0;
      uloz_mereni();
    }

  rotace_citac ++;
  if ((rotace_citac > 2) && (rotace_podtextu == true))   // rotujici text meni kazde 3 sekundy
    {
      rotace_citac = 0;

  /* indexpodtextu muze nabyvat hodnot:
    1 ... teplota
    2 ... baterie
    3 ... napeti 1   (jen kdyz je THUS_flag[1] = 1)
    4 ... napeti 2   (jen kdyz je THUS_flag[2] = 1)
    5 ... dalsi alarm    (jen kdyz je ala_count > 0)
  */

      indexpodtextu ++;
      if (indexpodtextu == 3 && thus_flag[1] == 0) indexpodtextu++;
      if (indexpodtextu == 4 && thus_flag[2] == 0) indexpodtextu++;
      if (indexpodtextu == 5 && ala_count == 0) indexpodtextu=1;
      if (indexpodtextu > 5) indexpodtextu = 1;



      switch (indexpodtextu)
        {
          case 1:    //zobrazeni teploty


            tep = teplota();
            tepkalib = 192;     // 192 = odecitat 1,5 stupne Celsia kvuli presnosti (128 = 1 stupen C)
            if (tep < 65535)      // pri odpojenem cidle: tep = 65535. V tom pripade se teplota nezobrazuje
              {
                lcdgoto05();
                if (tep > 32767 )       // zaporna teplota
                  {
                    lcd.LcdKod(45);     // znamenko minus "-"
                    tep = 65536 - tep + tepkalib ;  // pri zaporne hodnote vypocitat binarni doplnek + uparava o 1,5 stupne dolu
                  }
                else
                  {
                    lcd.LcdKod(43);     // pri kladne teplote se zobrazuje znamenko plus "+"
                    tep = tep - tepkalib;  // uprava o 1,5 stupne dolu
                  }

                tep = (tep >> 3);       // orezani stavovych bitu

                tepcitelna = (tep / 16.0);    // bezznamenkova cast teploty (desetinne cislo)
                lcd.LcdNum2(tepcitelna,2,1);
                kod32();            // mezera
                lcd.LcdKod(0x81);   // znacka stupen
                lcd.LcdString("C ");


              }
            else
              {
                rotace_citac = 3;
              }
            break;

          case 2:       // Stav baterie

            // Teorie prepoctu:
            //  Minimalni napeti na pinu Vin na Arduinu musi byt podle technicke specifikace 6V.
            //  Takze na baterii musi byt minimalne 6,4V (kvuli ubytku napeti na Schotkyho diode 0.4V).
            //  V serii mezi baterii a analogovym vstupem je Zenerova dioda 5V6.
            //  Pri minimalnim napeti 6,4V na baterii se na analogovy vstup dostane 6,4-5,6 = 0,8V
            //  0,8V na vstupu pri referencnim napeti 5V odpovida (0,8 / 5) * 1024 = 164 digitum
            //  Pro maximalni napeti na nove baterii (9,5V podle specifikace Energizer) je na vstupu 9,5-5,6 = 3,9V
            //  3,9V  na vstupu pri referencnim napeti 5V odpovida (3,9 / 5) * 1024 = 799 digitum
            //  Je treba provest linearizaci s posunutim do 0:    164 digitu odpovida 0%
            //                                                    799 digitu odpovida 100%
            //     Vysledny teoreticky vzorec pro vypocet procent je:
            //             batproc = ((batnap - 164) / (799 - 164)) * 100
            //                     = ((batnap - 164) /   6.35)

            // Podle skutecne namerenych hodnot vychazi vzorec:
            //             batproc = ((batnap - 132) / (772 - 132)) * 100
            //                     = ((batnap - 132) /   6.4)

            // promenna batproc je typu byte, takze automaticky orezava vysledek na cela cisla


            batnap = analogRead(A6);
            if (batnap < 132) batnap=132;    // pri odpojene baterii (pri napajeni z 12V nebo USB)
                                             // klesne napeti na analogovem vstupu na 0V, coz zpusobuje
                                             // nasledne "podteceni" promenne "batproc" pod 0 (pres 200%)

            if (batnap < 164 && batnap > 158 ) piip();    // pipani pri slabe baterii (mezi 4% a 5%)

            batproc =((batnap - 132) /  6.4);
            lcdgoto05();
            lcd.LcdString("BAT=");
            if (batproc == 0)
              {
                lcd.LcdString(" ---");
              }
            else
              {
                lcd.LcdNum(batproc,3,' ');
                lcd.LcdKod(37);                                   // znak procenta "%"
              }


            break;

          case 3:       // mereni napeti na A3
            lcdgoto05();
            lcd.LcdString("U1=");
            unap = analogRead(A3);
            vypoctene_cislo =((napetimax1 * unap) / 10230.0);            // tohle je treba upravit podle hardweru mereni napeti na rozsah 0V-5V
            lcd.LcdNum2(vypoctene_cislo,1,2);
            lcd.LcdString("V");
            break;

          case 4:       // mereni napeti na A7
            lcdgoto05();
            lcd.LcdString("U2=");
            unap = analogRead(A7);
            vypoctene_cislo =((napetimax2 * unap) / 10230.0);            // tohle je treba upravit podle hardweru mereni napeti na rozsah 0V-5V
            lcd.LcdNum2(vypoctene_cislo,1,2);
            lcd.LcdString("V");
            break;

          case 5:       // datum nebo cas dalsiho alarmu
            lcdgoto05();
            lcd.LcdString("A=");
            if ( precti_bajt( 9036 +(dalsi_alarm*64)) == mesdec && precti_bajt(9037 +(dalsi_alarm*64)) == dendec&& ((precti_bajt(9038 +(dalsi_alarm*64)) * 3600) + (precti_bajt(9039 +(dalsi_alarm*64)) * 60) + precti_bajt(9040 +(dalsi_alarm*64))) > ((hoddec * 3600) + (mindec * 60) + sekdec))
              {
                kod32();    // mezera pred hodinama
                lcd.LcdNum(precti_bajt( 9038 +(dalsi_alarm * 64)),2,'0');
                kod58();    // dvotecka ":" mezi hodinou a minutou
                lcd.LcdNum(precti_bajt( 9039 +(dalsi_alarm*64)),2,'0');
              }
            else
              {
                lcd.LcdNum(precti_bajt( 9037 +(dalsi_alarm*64)),2,' ');
                kod46();    //tecka "." mezi dnem a mesicem
                lcd.LcdNum(precti_bajt( 9036 +(dalsi_alarm*64)),2,' ');
                kod46();    //tecka "." za mesicem
              }
            break;

        }






    }


  lcd.big7seg(hoddes , 1);   // zobrazeni velkych cislic HH:MM   SS
  lcd.big7seg(hodjed , 2);
  lcd.big7seg(mindes , 3);
  lcd.big7seg(minjed , 4);
  lcd.dvojtecka();

  lcd.mid7seg(sekdes - '0',1);
  lcd.mid7seg(sekjed - '0',2);

}

void sipecka(byte indexmenu)
{

  for (i=0 ; i<6 ; i++)
    {
      if (i == indexmenu)
        {
          lcd.GotoXY(0, i);
          lcd.LcdKod(62);   // sipecka vpravo ">"
        }
      else
        {
          lcd.GotoXY(0, i);
          kod32();   //mezera
        }
    }


}





void sipecka2(byte indexmenu)
{
  byte ysip;

  for (i=0 ; i<4 ; i++)
    {

     if (i==0)
       {
         ysip=0;
       }
     else
       {
         ysip=(i*2)-1;
       }

      if (i == indexmenu)
        {
          lcd.GotoXY(0, ysip);
          lcd.LcdKod(62);   // sipecka vpravo ">"
        }
      else
        {
          lcd.GotoXY(0, ysip);
          kod32();   //mezera
        }
    }


}




//  =============== podprogramy pro cteni a zapis alarmu do externi SRAM ===========
void precti_alarm(byte alarmindex)     // vystupem jsou tri nastavene globalni promenne pro alarm (cas, spec a koment)
  {
    word start = ((alarmindex-1)*64) + 9100 ;
    for (adresa = start ; adresa < start + 5; adresa++)
      {
        alarm_cas[adresa-start] =  precti_bajt(adresa);
      }

    start = ((alarmindex-1)*64)+5 + 9100;
    alarm_spec =  precti_bajt(adresa);

    start=start+3;

    for (adresa = start ; adresa < start + 56; adresa++)
      {
        alarm_koment[adresa-start] =  precti_bajt(adresa);
      }
  }






byte najdi_dalsi_alarm(void)
{
  dalsi_alarm = 100 ;

  rtc_do_datcas();  // aktualni cas v RTC se prepise do 7 globalnich desitkovych promennych (rokdec, mesdec, dendec ..... mscdec)
  aktcas = (mesdec * 100000000UL) +  (dendec * 1000000UL) +  (hoddec * 10000UL) + (mindec * 100UL) + sekdec;

  unsigned long minrozdil = 0xFFFFFFFFUL;
  unsigned long maxrozdil = 0UL;

  ala_count = 0;
  for (i=1;i < ala_max+1 ; i++)
    {
      precti_alarm(i);
      if (alarm_spec > 127)     // berou se jen aktivni alarmy
        {
          ala_count ++;
          longalarm = (alarm_cas[0] * 100000000UL) + (alarm_cas[1] * 1000000UL) + (alarm_cas[2] * 10000UL) + (alarm_cas[3] * 100UL) + alarm_cas[4];
          unsigned long rozdil = longalarm - aktcas;

          if (longalarm < aktcas)
            {
              rozdil    = longalarm - aktcas + 1231235959UL;
              longalarm = longalarm + 1231235959UL;
            }

          if (minrozdil > rozdil )
            {
               dalsi_alarm = i;
               cas_dalsiho_alarmu = longalarm * 1ul;
               minrozdil = rozdil;
            }

          if (maxrozdil < rozdil )
            {
               predchozi_alarm = i;
               maxrozdil = rozdil;
            }



        }

    }


  zapis_bajt(7, ala_count); // zapis pocet aktivnich alarmu

  if (dalsi_alarm < 100)
    {
      return dalsi_alarm;
    }


}


void zrus_sipky(boolean CasFLAG)
{
  if (CasFLAG == true)
    {
      lcd.GotoXY(3, 5);
      kod32();     // mezera " "
      lcd.GotoXY(21, 5);
      kod32();     // mezera " "
      lcd.GotoXY(46, 5);
      kod32();     // mezera " "
      lcd.GotoXY(72, 5);
      kod32();     // mezera " "
    }
  else
    {
      lcd.GotoXY(3, 1);
      kod32();     // mezera " " na radu dni
      lcd.GotoXY(21, 1);
      kod32();     // mezera " " na radu mesicu
      lcd.GotoXY(60, 1);
      kod32();     // mezera " " na radu [X]
    }
  lcd.GotoXY(3, 3);
  kod32();     // mezera " " na radu hodin
  lcd.GotoXY(21, 3);
  kod32();     // mezera " " na radu minut
  lcd.GotoXY(39, 3);
  kod32();     // mezera " " na radu sekund



}


void zmenacasu()
{

  cas = rtc.formatTime(1);
  hoddes = cas[0];
  hodjed = cas[1];
  mindes = cas[2];
  minjed = cas[3];
  sekdes = cas[4];
  sekjed = cas[5];

  // setiny sekund se needituji

  dendes = cas[8];
  denjed = cas[9];
  mesdes = cas[10];
  mesjed = cas[11];
  rokdes = cas[12];
  rokjed = cas[13];


  cas_param[0] = (hoddes - '0') * 10 + (hodjed - '0');
  cas_param[1] = (mindes - '0') * 10 + (minjed - '0');
  cas_param[2] = (sekdes - '0') * 10 + (sekjed - '0');

  cas_param[3] = (dendes - '0') * 10 + (denjed - '0');
  cas_param[4] = (mesdes - '0') * 10 + (mesjed - '0');
  cas_param[5] = (rokdes - '0') * 10 + (rokjed - '0');

  cas_param[6] = 0; // neni aktivni storno

  lcdgoto02();
  lcd.LcdNum(cas_param[0], 2 , ' '); //hodina

  kod58();   // dvojtecka  ":"
  lcd.LcdNum(cas_param[1], 2 , '0');  //minuta

  kod58();   // dvojtecka  ":"
  lcd.LcdNum(cas_param[2], 2 , '0');  //sekunda

  lcdgoto04();
  lcd.LcdNum(cas_param[3], 2 , ' '); // den

  kod46();   // tecka  "."
  lcd.LcdNum(cas_param[4], 2 , ' '); // mesic

  kod46();   // tecka  "."
  lcd.LcdNum(cas_param[5] + 2000, 4 , '0'); // rok (2000 az 2099)

  lcd.LcdString(" [ ]");


  delpauza = 300;


  kurzor_x = 0;
  kurzor_y = 3;
  x_edit = 0;

  lcd.GotoXY(3, 3);
  kod0x80(); // plna sipka nahoru


  minipauza=millis();

  while (digitalRead(10) == HIGH && Serial.available() == 0)
    {

      if (minipauza + 180000 < millis()) jmp0();

      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);

      if (joystick_x >800)
        {
          kurzor_x --;
          if ( kurzor_x < 0 ) kurzor_x = 0;
          if ( kurzor_x < 3 ) kurzor_y = 3;
          delay(200+delpauza);
          delpauza=0;
        }

      if (joystick_x <100)
        {
          kurzor_x ++;
          if ( kurzor_x > 6 ) kurzor_x = 6;
          if ( kurzor_x > 2 ) kurzor_y = 5;
          delay(200+delpauza);
          delpauza=0;
        }


      if ((joystick_x <100) ||  (joystick_x >800))
        {
          minipauza=millis();
          zrus_sipky(true);

          switch (kurzor_x)
            {
              case 0:  // hodiny
                x_edit = 0;    // zacatek cisla hodin na displeji
                lcd.GotoXY(3, 3);
                kod0x80(); // plna sipka nahoru
                break;

              case 1:  // minuty
                x_edit = 18;    // zacatek cisla minut na displeji
                lcd.GotoXY(21, 3);
                kod0x80(); // plna sipka nahoru
                break;

              case 2:  // sekundy
                x_edit = 36;    // zacatek cisla sekund na displeji
                lcd.GotoXY(39, 3);
                kod0x80(); // plna sipka nahoru
                break;

              case 3:  // dny
                x_edit = 0;    // zacatek cisla dni na displeji
                lcd.GotoXY(3, 5);
                kod0x80(); // plna sipka nahoru
                break;

              case 4:  // mesice
                x_edit = 18;    // zacatek cisla mesicu na displeji
                lcd.GotoXY(21, 5);
                kod0x80(); // plna sipka nahoru
                lcd.GotoXY(46, 5);
                break;

              case 5:  // roky
                x_edit = 36+12;    // zacatek cisla roku na displeji
                lcd.GotoXY(46, 5);
                kod0x80(); // plna sipka nahoru
                break;

              case 6:  // STORNO
                x_edit = 36+12+7+5;    // zacatek napisu [X] na displeji
                lcd.GotoXY(72, 5);
                kod0x80(); // plna sipka nahoru
                break;



            }
        }


      if ((joystick_y < 800) && (joystick_y > 100) && (joystick_y < 800) && (joystick_y > 100))
        {
          delpauza=300;
        }
      else
        {
          minipauza=millis();
        }


      if (joystick_y > 800)
        {
          cas_param[kurzor_x] --;

         switch (kurzor_x)
          {
            case 0:  // hodiny
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 0;}
              break;
            case 1:  // minuty
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 59;}
              break;
            case 2:  // sekundy
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 59;}
              break;
            case 3:  // dny
              if (cas_param[kurzor_x] < 1) {cas_param[kurzor_x] = 1;}
              break;
            case 4:  // mesice
              if (cas_param[kurzor_x] < 1) {cas_param[kurzor_x] = 1;}
              break;
            case 5:  // roky
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 99;}
              break;
            case 6:  // storno  mze byt jen 0 nebo 1
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 0;}
              break;
          }




          delay(200+delpauza);
          delpauza=0;
        }

      if (joystick_y <100)
        {
         cas_param[kurzor_x] ++;
         switch (kurzor_x)
          {
            case 0:  // hodiny
              if (cas_param[kurzor_x] > 23) {cas_param[kurzor_x] = 23;}
              break;
            case 1:  // minuty
              if (cas_param[kurzor_x] > 59) {cas_param[kurzor_x] = 0;}
              break;
            case 2:  // sekundy
              if (cas_param[kurzor_x] > 59) {cas_param[kurzor_x] = 0;}
              break;
            case 3:  // dny
              if (cas_param[kurzor_x] > 31) {cas_param[kurzor_x] = 31;}  // bez kontroly korektniho zadani 28/29/30/31 dni v mesici
              break;
            case 4:  // mesice
              if (cas_param[kurzor_x] > 12) {cas_param[kurzor_x] = 12;}
              break;
            case 5:  // roky
              if (cas_param[kurzor_x] > 99) {cas_param[kurzor_x] = 0;}
              break;
            default:  // storno  muze byt jen 0 nebo 1
              if (cas_param[kurzor_x] > 1) {cas_param[kurzor_x] = 1;}
              break;


          }

          delay(200+delpauza);
          delpauza=0;
        }

       lcd.GotoXY(x_edit, kurzor_y -1);


       if ((kurzor_x == 0) || (kurzor_x == 3) || (kurzor_x == 4))
         {
           lcd.LcdNum(cas_param[kurzor_x], 2 , ' ');
         }
       else if ((kurzor_x == 1) || (kurzor_x == 2) || (kurzor_x == 5))
         {
           lcd.LcdNum(cas_param[kurzor_x], 2 , '0');
         }
       else if (kurzor_x == 6)
         {

           if (cas_param[kurzor_x] == 0) lcd.LcdString(" [ ]");
           else                          lcd.LcdString(" [X]");

         }



    }

  if (Serial.available()>0)
    {
      x_edit = Serial.read();
      if (x_edit = 67)                   // znak "C" = nastaveni jednotlivych casovych bunek
        {
          delay100();
          cas_param[0]=  Serial.read();  //hodina
          cas_param[1]=  Serial.read();  //hminuta
          cas_param[2]=  Serial.read();  //sekunda

          cas_param[3]=  Serial.read();  //den
          cas_param[4]=  Serial.read();  //mesic
          cas_param[5]=  Serial.read();  //rok
          while (Serial.available() == 0)
            {
              delay(1);
            }
          piip();

        }
    }




  // prevod na cas a ulozeni do RTC
  if (cas_param[6] == 0)
    {
      if (cas_param[3] == 31 and (cas_param[4]== 4 || cas_param[4]== 6 || cas_param[4]== 9 || cas_param[4]== 11))
          {
            cas_param[3] = 30;
          }
      if (cas_param[3] > 29 and (cas_param[4]== 2))
          {
            cas_param[3] = 29;
          }


      rtc.setTime(cas_param[0],cas_param[1],cas_param[2],cas_param[3],cas_param[4],cas_param[5]);
      najdi_dalsi_alarm();
    }

  while (Serial.available() > 0)  // jen vycisteni komunikacniho bufferu v pripade, ze byla provedena pocitacova synchronizace
    {
      x_edit = Serial.read();
    }

  lcd.LcdClear();
  delay200();








}



void zaznamcasu(byte smer_j)
{
  detachInterrupt(0);  // pri vychyleni joysticku a zaznamu casu se pozastavuje sekundovy interrupt pro zobrazeni casu na displeji

  if ( bzuk_flag[2]==1)  tone(A2, 800, 100);

  zapis_bajt(1,zaz_count >> 8);       // do prvni a druhe bunky externi RAM pameti zapise aktualni pocet zaznamu 1 az 999
  zapis_bajt(2,zaz_count & 0x00FF);

  lcd.ramecek(ala_count , zaz_count , mem_proc, inverzem);  // aktualizuje inforamecek

  rtc_do_datcas();  // aktualni cas v RTC se prepise do 7 globalnich desitkovych promennych (rokdec, mesdec, dendec ..... mscdec)

  // zapis aktualniho casu do bunek externi pameti RAM
  adresa = (zaz_count * 9) + 11;   // zacina se prvnim blokem zaznamu od 11. bajtu
  zapis_bajt (adresa    ,rokdec);
  zapis_bajt (adresa + 1,mesdec);
  zapis_bajt (adresa + 2,dendec);
  zapis_bajt (adresa + 3,hoddec);
  zapis_bajt (adresa + 4,mindec);
  zapis_bajt (adresa + 5,sekdec);
  zapis_bajt (adresa + 6,mscdec);

  zapis_bajt (adresa + 7,smer_j);   // smer nakloneni joysticku

  lcdgoto05();               // zapis casu zaznamu pod inforamecek  H:MM:SS
  lcd.LcdNum(hoddec, 2 , ' ');
  kod58();          // dvojtecka mezi hodinami a minutami
  lcd.LcdNum(mindec, 2 , '0');
  kod58();          // dvojtecka mezi minutami a sekundami
  lcd.LcdNum(sekdec, 2 , '0');
  kod32();          // mezera na doplneni radky na 9 znaku


  rotace_podtextu = false;

  minipauza = millis();
  delay100();

  attachInterrupt(0, preruseni, FALLING);  // opetovne spusteni sekundoeho preruseni pro aktualizaci casu na displeji

}


// zapis alarmu bez komentare (pouzito pri editaci alarmu, kdyz se komentar nemeni)
void zapis_alarm(byte alarmindex , byte* cas , byte spec)
  {
    int i;

    adresa = (alarmindex-1)*64 + 9100;
    for (i = 1; i < 6 ; i++)
      {
        zapis_bajt(adresa , cas[i]);
        adresa ++;
      }

    zapis_bajt(adresa , spec);
  }


// zapis alarmu s komentarem (pouzity pri nahrani alarmu z PC, nebo pri tovarnim nastaveni)
void zapis_alarm(byte alarmindex , byte* cas , byte spec , char* koment)
{
  int i;

  adresa = (alarmindex-1)*64 + 9100;
  for (i = 1; i < 6 ; i++)
  {
    zapis_bajt(adresa , cas[i]);
    adresa ++;
  }

  zapis_bajt(adresa , spec);
  adresa = adresa + 3;

  for (i = 0; i < 56 ; i++)
    {
      zapis_bajt(adresa , koment[i]);
      adresa ++;
    }
}

// ================= konec podprogramu pro alarmy v pridavne SRAM =====================


// na displeji se zobrazi alarm vcetne casu a textoveho popisu
void ukaz_alarmy(byte alarm_index,boolean ukazrok)
{

  byte vypocteny_rok;

  precti_alarm(alarm_index);
  lcd.LcdClear();
  lcdgoto00();     // na horni radku poskladat datum  " D. M."
  lcd.LcdNum(alarm_cas[1],2,' ');     // den
  kod46();     // tecka "."
  lcd.LcdNum(alarm_cas[0],2,' ');     // mesic
  kod46();     // tecka "."


  // urceni, pro ktery rok je alarm platny (pro letosni, nebo pro pristi). Rok se do alarmu v RTC normalne neukalda
  rtc_do_datcas();  // aktualni cas v RTC se prepise do 7 globalnich desitkovych promennych (rokdec, mesdec, dendec ..... mscdec)



  // prevod na desitkovy (lehce citelny) format mmddhhnnss
  long aktcas =         (mesdec * 100000000L) +         (dendec * 1000000L) +        (hoddec * 10000L) +         (mindec * 100L) +     sekdec;
  long longalarm =    (alarm_cas[0] * 100000000L) +   (alarm_cas[1] * 1000000L) +  (alarm_cas[2] * 10000L) +  (alarm_cas[3] * 100L) + alarm_cas[4];

  if (aktcas < longalarm) vypocteny_rok = rokdec;
  else  vypocteny_rok = rokdec+1;

  if (ukazrok == true)
    {
      lcd.LcdString("20");                  // prvni dvojcisli vypocteneho ctyrmistneho roku
      lcd.LcdNum(vypocteny_rok,2,'0');     // druhe dvojcisli roku
    }

  lcd.GotoXY(66, 0);        // posun na pozici hranate zavorky
  lcd.LcdString("[ ]");     // tecka za mesicem a disablovaci policko
  if (alarm_spec > 127)  // kdyz je nastaveny nejvyssi bit, nastavi se zaskrtavatko "A"
    {
      lcd.GotoXY(72, 0);     // posun na pozici krizku
      lcd.LcdKod(65);        // "A" = aktivni
    }



  lcdgoto01();                                   // na druhou radku poskladat cas " H:MM:SS"
  lcd.LcdNum(alarm_cas[2],2,' ');     // hodina
  kod58();     // dvojtecka ":"
  lcd.LcdNum(alarm_cas[3],2,'0');     // minuta
  kod58();     // dvojtecka ":"
  lcd.LcdNum(alarm_cas[4],2,'0');     // sekunda

  lcd.GotoXY(54, 1);                                   // pozice pro zobrazeni indexu v pameti
  lcd.LcdString("i=");
  lcd.LcdNum(alarm_index,2,' ');     // aktualni index alarmu v pameti


  lcdgoto02();              // zobrazeni textoveho popisu alarmu (vychazi to presne na zbytek plochy displeje - 56 znaku = 14 sloupcu * 4 radky)
 // lcd.LcdString(alarm_koment);


    for (i=0;i<56;i++)              // zobrazi komentar (vsech 56 znaku)
      {
        lcd.LcdKod((byte)alarm_koment[i]);
      }


}







void edituj_alarm(byte n)
{
  do
    {
      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);
      delay100();
    }
    while ((joystick_y>800) || (joystick_y<100));




  precti_alarm(n);
  lcd.LcdClear();
  lcdgoto00();     // na horni radku poskladat datum  " D. M."
  lcd.LcdNum(alarm_cas[1],2,' ');     // den
  kod46();     // tecka "."
  lcd.LcdNum(alarm_cas[0],2,' ');     // mesic
  lcd.LcdString(".   [A]");   // tecka za mesicem a policko s aktivnim alarmem
  cas_param[2] = 128;
  if (alarm_spec < 128)    // kdyz neni nastaveny nejvyssi bit, alarm se ignoruje
    {
      cas_param[2] = 127;      // neaktivni alarm
      lcd.GotoXY(60, 0);     // posun na pozici krizku
      kod32();        // " " mezera znamena neaktivni alarm
    }

  lcdgoto02();                                   // na treti radku poskladat cas " H:MM:SS"
  lcd.LcdNum(alarm_cas[2],2,' ');     // hodina
  kod58();     // dvojtecka ":"
  lcd.LcdNum(alarm_cas[3],2,'0');     // minuta
  kod58();     // dvojtecka ":"
  lcd.LcdNum(alarm_cas[4],2,'0');     // sekunda


  cas_param[0] = alarm_cas[1]; // den
  cas_param[1] = alarm_cas[0]; // mesic
  cas_param[3] = alarm_cas[2]; // hodina
  cas_param[4] = alarm_cas[3]; // minuta
  cas_param[5] = alarm_cas[4]; // sekunda



  lcdgoto04();                                   // na treti radku poskladat cas " H:MM:SS"
  for (i=0;i<28;i++)              // zobrazi prvnich 28 znaku z komentare
  {
    lcd.LcdKod((byte)alarm_koment[i]);
  }


  delpauza = 300;


  kurzor_x = 0;  // index kurzoru X (0 az 5)
  kurzor_y = 1;
  x_edit = 0;    // pozice X, na ktere se budou zobrazovat nastavene hodnoty

  lcd.GotoXY(3, 1);
  kod0x80(); // plna sipka nahoru na nilte pozici kurzoru (dny)


  while (digitalRead(10) == HIGH)
    {

      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);

      if (joystick_x >800)
        {
          kurzor_x --;
          if ( kurzor_x < 0 ) kurzor_x = 0;
          if ( kurzor_x < 3 ) kurzor_y = 1;
          delay(200+delpauza);
          delpauza=0;
        }

      if (joystick_x <100)
        {
          kurzor_x ++;
          if ( kurzor_x > 5 ) kurzor_x = 5;
          if ( kurzor_x > 2 ) kurzor_y = 3;
          delay(200+delpauza);
          delpauza=0;
        }


      if ((joystick_x <100) ||  (joystick_x >800))
        {
          zrus_sipky(false);
          switch (kurzor_x)
            {
              case 0:  // dny
                x_edit = 0;    // zacatek cisla dni na displeji
                lcd.GotoXY(3, 1);
                kod0x80(); // plna sipka nahoru na pozici dni
                break;

              case 1:  // mesice
                x_edit = 18;    // zacatek cisla mesicu na displeji
                lcd.GotoXY(21, 1);
                kod0x80(); // plna sipka nahoru
                break;

              case 2:  // [X]
                x_edit = 60;    // zacatek zaku X n displeji
                lcd.GotoXY(60, 1);
                kod0x80(); // plna sipka nahoru
                break;

              case 3:  // hodiny
                x_edit = 0;    // zacatek cisla hodin na displeji
                lcd.GotoXY(3, 3);
                kod0x80(); // plna sipka nahoru
                break;

              case 4:  // minuty
                x_edit = 18;    // zacatek cisla minut na displeji
                lcd.GotoXY(21, 3);
                kod0x80(); // plna sipka nahoru
                break;

              default:  // sekundy
                x_edit = 36;    // zacatek cisla roku na displeji
                lcd.GotoXY(39, 3);
                kod0x80(); // plna sipka nahoru
                break;


            }
        }


      if ((joystick_y < 800) && (joystick_y > 100) && (joystick_x < 800) && (joystick_x > 100))
        {
          delpauza=300;
        }


      if (joystick_y > 800)
        {
          cas_param[kurzor_x] --;

         switch (kurzor_x)
          {
            case 0:  // dny
              if (cas_param[kurzor_x] < 1) {cas_param[kurzor_x] = 1;}
              break;
            case 1:  // mesice
              if (cas_param[kurzor_x] < 1) {cas_param[kurzor_x] = 1;}
              break;
            case 2:  // [V]
              if (cas_param[kurzor_x] < 127) {cas_param[kurzor_x] = 127;}
              break;
            case 3:  // hodiny
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 0;}
              break;
            case 4:  // minuty
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 59;}
              break;
            default:  // sekundy
              if (cas_param[kurzor_x] < 0) {cas_param[kurzor_x] = 59;}
              break;
          }




          delay(200+delpauza);
          delpauza=0;
        }

      if (joystick_y <100)
        {
         cas_param[kurzor_x] ++;
         switch (kurzor_x)
          {
            case 0:  // dny
              if (cas_param[kurzor_x] > 31) {cas_param[kurzor_x] = 31;}
              break;
            case 1:  // mesice
              if (cas_param[kurzor_x] > 12) {cas_param[kurzor_x] = 12;}
              break;
            case 2:  // [V]
              if (cas_param[kurzor_x] > 128) {cas_param[kurzor_x] = 128;}
              break;
            case 3:  // hodiny
              if (cas_param[kurzor_x] > 23) {cas_param[kurzor_x] = 23;}
              break;
            case 4:  // minuty
              if (cas_param[kurzor_x] > 59) {cas_param[kurzor_x] = 0;}
              break;
            default:  // sekundy
              if (cas_param[kurzor_x] > 59) {cas_param[kurzor_x] = 0;}
              break;


          }

          delay(200+delpauza);
          delpauza=0;
        }

       lcd.GotoXY(x_edit, kurzor_y -1);


       if ((kurzor_x == 0) || (kurzor_x == 1) || (kurzor_x == 3))
         {
           lcd.LcdNum(cas_param[kurzor_x], 2 , ' ');
         }
       else if ((kurzor_x == 4) || (kurzor_x == 5))
         {
           lcd.LcdNum(cas_param[kurzor_x], 2 , '0');
         }
       else if (kurzor_x == 2)
         {

           if (cas_param[2] == 127) kod32();  // znak mezera " " = neaktivni alarm
           else                     lcd.LcdKod(65);  // znak "A" = aktivni

         }
      }


      if (cas_param[0] == 31 and (cas_param[1]== 4 || cas_param[1]== 6 || cas_param[1]== 9 || cas_param[1]== 11))
          {
            cas_param[0] = 30;
          }
      if (cas_param[0] > 29 and (cas_param[1]== 2))
          {
            cas_param[0] = 29;
          }



      alarm_spec   =  cas_param[2];
      alarm_cas[1] =  cas_param[1];         // mesic
      alarm_cas[2] =  cas_param[0];         // den
      alarm_cas[3] =  cas_param[3];         // hodina
      alarm_cas[4] =  cas_param[4];         // minuta
      alarm_cas[5] =  cas_param[5];         // sekunda



      zapis_alarm(n,alarm_cas ,alarm_spec);
      najdi_dalsi_alarm();   // aktualizuje pristi alarm a zaroven zapise pocet aktivnich alarmu do SRAM



  delay(1000);
  ukaz_alarmy(n,false);
}



void read_eeprom_txt(word adresa, byte delka)
{
   i = 0;
   while (i < delka)
     {
       lcd.LcdKod(EEPROM.read(adresa + i));
       i++;
     }
}



void ukaz_zaznamy()
{
  lcd.LcdClear();
  if (zaz_count == 0)
    {
      lcd.GotoXY(6, 2);
      lcd.LcdString("Zadny zaznam");
      delay500();
      return;
    }

  zaz_index = zaz_count;

  ukazzaznam = true;
  minipauza = millis();

  while (digitalRead(10) == HIGH)
    {
      joystick_x = analogRead(A1);
      joystick_y = analogRead(A0);

      if (minipauza + 120000 < millis()) jmp0();

      if (joystick_x > 800)
        {
          minipauza = millis();
          zaz_index --;
          if (zaz_index < 1) zaz_index = 1;
          ukazzaznam = true;
          delay(300);
        }
      if (joystick_x < 100)
        {
          minipauza = millis();
          zaz_index ++;
          if (zaz_index > (zaz_count+1)) zaz_index = zaz_count + 1;
          ukazzaznam = true;
          delay(300);
        }

      if (joystick_y > 800)
        {
          minipauza = millis();
          zaz_index = zaz_index - 20;
          if ((zaz_index < 1) || (zaz_index > 2000) ) zaz_index = 1;
          ukazzaznam=true;
          delay(300);
        }
      if (joystick_y < 100)
        {
          minipauza = millis();
          zaz_index = zaz_index + 20;
          if (zaz_index > zaz_count+1) zaz_index = zaz_count+1;
          ukazzaznam=true;
          delay(300);
        }

      if ((ukazzaznam == true) && (zaz_index <= zaz_count))
        {
          adresa = (zaz_index * 9) + 11;
          lcdgoto01();
          read_eeprom_txt(994,6);  // text "Index:"
          lcd.LcdNum(zaz_index,4,' ');  // index
          lcdgoto02();
          lcd.LcdNum(precti_bajt(adresa + 2),2,' ');  // den
          lcd.LcdString(".");
          lcd.LcdNum(precti_bajt(adresa + 1),2,' ');  // mesic
          lcd.LcdString(".20");
          lcd.LcdNum(precti_bajt(adresa),2,'0');      // rok
          lcd.LcdString("    ");

          lcd.GotoXY(0, 3);
          lcd.LcdNum(precti_bajt(adresa + 3),2,' ');  // hodina
          kod58();     // dvojtecka ":"
          lcd.LcdNum(precti_bajt(adresa + 4),2,'0');  // minuta
          kod58();     // dvojtecka ":"
          lcd.LcdNum(precti_bajt(adresa + 5),2,'0');  // sekunda
          kod46();     // tecka "."
          lcd.LcdNum(precti_bajt(adresa + 6),2,'0');  // setiny sekundy

          lcdgoto04();
          lcd.LcdString("dir=");
          lcd.LcdKod(precti_bajt(adresa + 7));  // smer (N/D/L/P)
          lcd.LcdString(" dur=");
          lcd.LcdNum((precti_bajt(adresa + 8)/10),2,'0');  // trvani - cele sekundy
          kod46();     // tecka "."
          lcd.LcdNum((precti_bajt(adresa + 8)%10),1,'0');  // trvani - desetiny sekund



          ukazzaznam = false;
        }

      if ((ukazzaznam == true) && (zaz_index > zaz_count))
        {

          lcd.LcdClear();
          lcd.GotoXY(24, 2);

                  //  *    Smazat    *
                  //  *   zaznamy?   *
          lcd.LcdString("Smazat      zaznamy?");

          ukazzaznam = false;
        }


    }

  if (zaz_index == zaz_count+1)
    {

          lcd.GotoXY(6, 2);

                  //  * Zaznamy byly *
                  //  *   smazany!   *

          lcd.LcdString("Zaznamy byly    smazany!");
          // ve skutecnosti se zaznamy nemazou, ale jen se nastavi ukazatel na 1. zaznam a pri dalsim ukladani se stare zaznamy prepisuji
          // za urcitych okolnosti je tedy mozne i "smazane" zaznamy obnovit (pokud jeste nejsou prepsany)
          zaz_count = 0;
          zapis_bajt(1,0);       // vynulovani poctu zaznamu v pameti (adresa 1 a 2)
          zapis_bajt(2,0);
          delay(1000);

    }
}



void zmenajasu()
{

  delay200();

  lcdgoto02();
  lcd.LcdString("Hodnota:");

  minipauza=millis();

  while (digitalRead(10) == HIGH)
    {

      if (minipauza +120000 < millis()) jmp0();

      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);

      if (joystick_y > 800)
        {
          minipauza=millis();
          jas = jas - 20;
          if ( jas < 0 ) jas = 0;

          lcd.podsvet(jas);
          delay200();
        }

      if (joystick_y < 100)
        {
          minipauza=millis();
          jas = jas + 20;
          if ( jas > 255 ) jas = 255;

          lcd.podsvet(jas);
          delay200();
        }


      if (joystick_x > 800)
        {
          minipauza=millis();
          jas --;
          if ( jas < 0 ) jas = 0;

          lcd.podsvet(jas);
          delay500();
        }

      if (joystick_x < 100)
        {
          minipauza=millis();
          jas ++;
          if ( jas > 255 ) jas = 255;
          lcd.podsvet(jas);
          delay500();
        }


      lcd.GotoXY(50, 2);
      lcd.LcdNum(jas,3,' ');

    }

  zapis_bajt(3, (jas));
  delay200();

}



void listuj_alarmama(byte startindex_alarmu)
{
  zaz_index = startindex_alarmu;
  ukazzaznam = true;
  minipauza = millis();
  while (digitalRead(10) == HIGH)
    {
      joystick_x = analogRead(A1);
      joystick_y = analogRead(A0);
      if (minipauza + 120000 < millis())  jmp0();

      if (joystick_x > 800)
        {
          minipauza = millis();
          zaz_index --;
          if (zaz_index < 1)
            {
              zaz_index = 1;
            }
          else
            {
              ukazzaznam = true;
              delay(300);
            }
        }
      if (joystick_x < 100)
        {
          minipauza = millis();
          zaz_index ++;
          if (zaz_index > ala_max)
            {
              zaz_index = ala_max;
            }
          else
            {
              ukazzaznam = true;
              delay(300);
            }
        }

      if (joystick_y <100) // editace alarmu po naklonu joysticku nahoru
        {
          edituj_alarm(zaz_index);
        }

      if ((ukazzaznam == true) && (zaz_index <= ala_max))
        {
          ukaz_alarmy(zaz_index,true);
          ukazzaznam = false;
        }
    }


}


void tovarni_nastaveni()
{
  char * alarm_koment;    // 56-znakovy komentar ke kazdemu alarmu

  zapis_bajt(1, 0);       // pocet zaznamu casu HIGH
  zapis_bajt(2, 0);       // pocet zaznamu casu LOW

  zapis_bajt(3, 128);     // podsvet na stredni hodnotu (255 = maximum ; 0 = zhasnuto)

  zapis_bajt(4, 0x49);    // prvni volna bunka pro autozaznam HIGH (adresa 18701)
  zapis_bajt(5, 0x0D);    // prvni volna bunka pro autozaznam LOW

  zapis_bajt(6, 0b0110);  // nastaveni piskaku (posledni 4 bity)  hodinove NE / alarmove ANO / joystickove ANO / ukladaci NE

  zapis_bajt(7, 0);       // zapis pocet AKTIVNICH alarmu (po resetu do tovarniho nastaveni neni zadny aktivni)
  zapis_bajt(8, 5);       // zapis celkovy pocet alarmu (5 uzivatelskych)

  zapis_bajt(9, 0);       // perioda ukladani dat HIGH
  zapis_bajt(10, 60);     // perioda ukladani dat LOW (defaultne 60 sekund)

  zapis_bajt(11, 0b000);  // Co ukladat (T H U S) - teplota NE/ napeti 1 NE/ napeti 2 NE
  zapis_bajt(12, 0);      // kalibracni konstanta RTC na 0

  zapis_bajt(13,1);       // index nasledujiciho bloku ukladani T H U S ve FAT (1 az 200)

  zapis_bajt(14, 1);      // rychlost komunikace na 38400

  zapis_bajt(15, 205);    // VOP - parametr displeje Nokia (pouzito pro kontrast)
  zapis_bajt(16, 4);      // temperature - parametr displeje Nokia
  zapis_bajt(17, 17);     // Bias - parametr displeje Nokia

  zapis_bajt(18, 50);     // Rozsah prvniho napetoveho vstupu je 5,0V
  zapis_bajt(19, 50);     // Rozsah druheho napetoveho vstupu je 5,0V


  rtc.setKalib(0);        // kalibracni konstanta RTC obvodu na 0

  ala_max = 5;            // 5 uzivatelskych nesmazatelnych alarmu



  // kompletni vymazani cele SRAM od adresy 20 az do konce (zaznamy, alarmy, FAT, namerena data)
  for (adresa = 20; adresa < 65535 ; adresa ++)
    {
      zapis_bajt(adresa, 0);
    }


  for (i=1;i<6;i++)
    {
      // uzivatelske alarmy se nastavi na 1.1. 00:00:00 a znacka se nastavi na "neaktivni alarm"
      alarm_cas[1] = 0x01;         //  mesic
      alarm_cas[2] = 0x01;         //  den
      alarm_cas[3] = 0x00;         // hodina
      alarm_cas[4] = 0x00;         // minuta
      alarm_cas[5] = 0x00;         //  sekunda
      alarm_spec   = 0x00;         // nejvyssi bit je znacka aktivniho alarmu


      alarm_koment = "  Uzivatelsky    alarm c.                               ";
      alarm_koment[25] = '0' + i;

      zapis_alarm(i,alarm_cas,alarm_spec,alarm_koment);
    }




  // nastaveni RTC pri tovarnim resetu na 1.1.2000 00:00:00
  rtc.init();
  rtc.setTime(0,0,0,1,1,0);
  delay500();




  jmp0();    // softwarovy reset


}


void menu_1_cisla(boolean ukazcisla)
{
  byte pomprom = precti_bajt(6);
  bzuk_flag[0] = (pomprom >> 3);                 // H-odinove
  bzuk_flag[1] = ((pomprom & 0b00000100) >> 2);  // A-larmove
  bzuk_flag[2] = ((pomprom & 0b00000010) >> 1);  // J-oystickove
  bzuk_flag[3] =   pomprom & 0b00000001;         // S-ejfovaci

  pomprom = precti_bajt(11);
  thus_flag[0] = (pomprom  >> 2);                  // T-eplota
  thus_flag[1] = ((pomprom & 0b00000010) >> 1);    // U1 - prvni napeti
  thus_flag[2] =   pomprom & 0b00000001;           // U2 - druhe napeti


  perioda_save=(precti_bajt(9)<<8) + precti_bajt(10);
  kalibrace = precti_bajt(12);

  if (ukazcisla==true)
    {
      lcd.GotoXY(60, 0);
      if(kalibrace > 31 )
        {
          lcd.LcdKod(45);   // znak minus "-"
          lcd.LcdNum(32 - (kalibrace & 0b00011111),2,'0');
        }
      else
        {
          lcd.LcdKod(43);   // znak plus "+"
          lcd.LcdNum((kalibrace & 0b00011111),2,'0');
        }


      // zobrazeni H A J S flagu
      lcd.GotoXY(42, 1);
      lcd.LcdNum(bzuk_flag[0],1,'0');
      lcd.GotoXY(54, 1);
      lcd.LcdNum(bzuk_flag[1],1,'0');
      lcd.GotoXY(66, 1);
      lcd.LcdNum(bzuk_flag[2],1,'0');
      lcd.GotoXY(78, 1);
      lcd.LcdNum(bzuk_flag[3],1,'0');

      // zobrazeni T H U S flagu
      lcd.GotoXY(36, 3);
      lcd.LcdNum(thus_flag[0],1,'0');
      lcd.GotoXY(51, 3);
      lcd.LcdNum(thus_flag[1],1,'0');
      lcd.GotoXY(69, 3);
      lcd.LcdNum(thus_flag[2],1,'0');


      lcd.GotoXY(54, 5);
      lcd.LcdNum(perioda_save,3,'0');


      cas_param[0] = precti_bajt(12);            // kalibrace
      cas_param[1] = precti_bajt(6);            // piskak H A J S
      cas_param[2] = precti_bajt(11);            // THUS
      cas_param[3] = (precti_bajt(9) << 8) + precti_bajt(10); // perioda ukladani


    }

}


void menu_nastaveni1()
{
  boolean ukazcisla;
  detachInterrupt(0); // zastavit aktualizaci zobrazeni casu na displeji
  minipauza = millis();
  lcd.LcdClear();
  while (digitalRead(10) == LOW)   // dokud je stisknuty joystick nic se nedeje
    {
       delay200();       // osetreni zakmitani kontaktu
    }


  // zobrazeni menu
  lcd.GotoXY(6, 0);
  read_eeprom_txt(1000,7);  // text "RTC-kal"
  lcd.GotoXY(6, 1);
  lcd.LcdString("BZUK");
  lcd.GotoXY(42, 2);
  lcd.LcdString("H A J S");
  lcd.GotoXY(6, 3);
  lcd.LcdString("SAVE");
  lcd.GotoXY(36, 4);
  lcd.LcdString("T U1 U2");
  lcd.GotoXY(6, 5);
  read_eeprom_txt(987,7);  // text "Perioda"
  lcd.GotoXY(6, 78);
  lcd.LcdKod(115);          // znak "s" (sekundy)

  indexmenu = 0;   // indexmenu = 0 (prednastavena polozka JAS)
  lcdgoto00();
  lcd.LcdKod(62); // zobrazeni male sipecky ">" na prvni radce (polozka jas)


  menu_1_cisla(true);

  delpauza = 500;


  while ((digitalRead(10) == HIGH))
    {
      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);
      if (minipauza + 120000 < millis()) jmp0();
      if (joystick_y < 100)
        {
          minipauza = millis();
          indexmenu --;
          if ( indexmenu < 0 ) {indexmenu = 0;}
          sipecka2(indexmenu);
          delay500();
        }

      if (joystick_y > 800)
        {
          minipauza = millis();
          indexmenu ++;
          if ( indexmenu > 3 ) {indexmenu = 3;}
          sipecka2(indexmenu);
          delay500();
        }

      if (joystick_x > 800)
        {
          cas_param[indexmenu] --;
          if (cas_param[indexmenu] < 0) cas_param[indexmenu] = 0 ;
          switch (indexmenu)
            {
              case 0:
                zapis_bajt(12,cas_param[0]);
                break;

              case 1:
                zapis_bajt(6,cas_param[1]);
                break;

              case 2:
                zapis_bajt(11,cas_param[2]);
                break;

              case 3:
                if (cas_param[3] < 1) cas_param[indexmenu] = 1 ;
                zapis_bajt(9 ,cas_param[3]/256);
                zapis_bajt(10,cas_param[3]%256);
            }
          ukazcisla = true;
        }

      if (joystick_x < 100)
        {
          cas_param[indexmenu] ++;
          switch (indexmenu)
            {
              case 0:
                if (cas_param[0] > 63) cas_param[0] = 63;
                zapis_bajt(12,cas_param[0]);
                break;

              case 1:
                if (cas_param[1] > 15) cas_param[1] = 15;
                zapis_bajt(6,cas_param[1]);
                break;

              case 2:
                if (cas_param[2] > 7) cas_param[2] = 7;
                zapis_bajt(11,cas_param[2]);
                break;

              default:
                if (cas_param[3] > 999) cas_param[3] = 999;
                zapis_bajt(9 ,cas_param[3]/256);
                zapis_bajt(10,cas_param[3]%256);
            }
          ukazcisla = true;

        }

      if ((joystick_x < 800) && (joystick_x > 100))
        {
          ukazcisla = false;
          delpauza = 500;
        }

      if ((joystick_x > 800) || (joystick_x < 100))
        {
          minipauza = millis();
          menu_1_cisla(ukazcisla);
          delay(50 + delpauza);
          delpauza = delpauza - 100;
          if (delpauza < 0) delpauza = 0;
        }



    }

  rtc.setKalib(kalibrace);
  delay(1000);

}

void print_rych(byte komunrych)
{
  char* rychlosttxt;
  lcd.GotoXY(48, 2);
  switch (komunrych)
    {
      case 1:
        rychlosttxt = " 38.4";
        break;
      case 2:
        rychlosttxt = " 57.6";
        break;
      default:
        rychlosttxt = "  9.6";
        break;

    }

  lcd.LcdString(rychlosttxt);
  zapis_bajt(14,komunrych);

}



void menu_nastaveni2()
{
  detachInterrupt(0); // zastavit aktualizaci zobrazeni casu na displeji
  lcd.LcdClear();
  while (digitalRead(10) == LOW)   // dokud je stisknuty joystick nic se nedeje
    {
       delay200();       // osetreni zakmitani kontaktu
    }


  // zobrazeni menu
  lcd.GotoXY(6, 1);
  lcd.LcdString("Zpet");

  int komunrych = precti_bajt(14);
  lcd.GotoXY(6, 2);
  lcd.LcdString("Baud:");

  print_rych(komunrych);


  int kontrast = precti_bajt(15);
  if ( kontrast < 128) kontrast = 192;
  lcd.GotoXY(6, 3);
  lcd.LcdString("Kontrast:");
  lcd.LcdNum(kontrast,3,'0');

  lcd.GotoXY(6, 4);
  lcd.LcdString("Format FAT");
  lcd.GotoXY(6, 5);
  read_eeprom_txt(1007,13);  // text "Tovarni RESET"



  indexmenu = 1;   // indexmenu = 1 (na horni radce)
  lcdgoto01();
  lcd.LcdKod(62); // zobrazeni male sipecky ">" na radce "ZPET"


  // dokud neni stisknuto tlacitko v joysticku, provadi sistovani po polozkach nahoru a dolu
  while ((digitalRead(10) == HIGH))
    {
      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);
      if (joystick_y < 100)
        {
          indexmenu --;
          if (indexmenu < 1) indexmenu=1;
          sipecka(indexmenu);
        }

      if (joystick_y > 800)
        {
          indexmenu ++;
          if (indexmenu > 5) indexmenu = 5;
          sipecka(indexmenu);
        }

      if (joystick_x <100 && indexmenu == 2)
        {
          komunrych ++;
          if (komunrych > 2) komunrych = 2;
          print_rych(komunrych);

        }

     if (joystick_x >800 && indexmenu == 2)
        {
          komunrych --;
          if (komunrych < 0) komunrych = 0;
          print_rych(komunrych);
        }


      if (joystick_x <100 && indexmenu == 3)
        {
          kontrast ++;
          if (kontrast > 255) kontrast = 255;
          lcd.GotoXY(60, 3);
          lcd.LcdNum(kontrast,3,'0');
          zapis_bajt(15,kontrast);
        }

      if (joystick_x >800 && indexmenu == 3)
        {
          kontrast --;
          if (kontrast < 0) kontrast = 0;
          lcd.GotoXY(60, 3);
          lcd.LcdNum(kontrast,3,'0');
          zapis_bajt(15,kontrast);
        }
      delay200();




    }

  lcd.LcdClear();
  delay200();

  lcdgoto00();   // smazani displeje a rozeskok na pozadovanou akci

  switch (indexmenu)
      {


        case 4:    // nedochazi k prepsani vsech namerenych hodnot, ani ke smazani FAT. Jen se nastavi ukazatele na zacatek.
          lcd.LcdString("Format FAT");
          zapis_bajt(4, 0x49); // prvni volna bunka pro autozaznam HIGH (adresa 18701)
          zapis_bajt(5, 0x0D); // prvni volna bunka pro autozaznam LOW
          zapis_bajt(13, 1);   // index nasledujiciho bloku ukladani T H U S ve FAT (1 az 200)
          mem_proc=99;
          delay(1000);
          break;


        case 5:  // tohle je ale opravdovy prepis vsech hodnot v SRAM defaultnimi udaji - vcetne kompletniho smazani prazdne SRAM
          lcd.LcdString("Tovarni  RESET");
          tovarni_nastaveni();
          break;
      }





  delay500();

  jmp0();

 // lcd.LcdClear();
 // attachInterrupt(0, preruseni, FALLING);



}

void print_crc(unsigned long crc_ardu)
{
  lcd.GotoXY(0,3);
  lcd.LcdString("CRC soucet:");
  lcd.GotoXY(0,4);
  lcd.LcdNum(crc_ardu,9,'0');
}

void komunikace(byte parametr)
{
  word maxdata;
  unsigned long crc_ardu;
  byte prichozi_byte;
  byte odchozi_byte;

  zapis_bajt(11,0);      // po navratu z komunikace se nebudou zaznamenavat zadne merene kanaly (THUS flag = 000)
  menu_1_cisla(false);   // prepocet z THUS = 0 na THUS[0] = 0 , THUS[1]=0, THUS[2]=0

  read_eeprom_txt(966, 6);    // napis "Prenos PC"  (oba smery)

  minipauza = millis();



  if (parametr == 0)
    {
      while (Serial.available() == 0)
         {
           delay(10);
           if (minipauza+120000 < millis() || digitalRead(10) == LOW) jmp0();
         }

      prichozi_byte = Serial.read();
    }
  else
    {
      prichozi_byte = parametr;
    }

  delay500();
  Serial.println(123);
  delay100();

  lcd.GotoXY(78,5);
  lcd.LcdKod(37);    // znak procenta "%" na pravy dolni roh displeje

  crc_ardu =0UL;
  lcdgoto01();

   if (prichozi_byte == 81)    // znak Q - download z hodin do PC
     {
        lcd.LcdString("Hodiny -> PC");
        maxdata=(precti_bajt(4)<<8) + precti_bajt(5);  // prvni volny bajt za namerenymi hodnotami
        for (adresa=0;adresa < maxdata;adresa++)
          {
            odchozi_byte = precti_bajt(adresa);
            crc_ardu = crc_ardu + odchozi_byte;
            Serial.println(odchozi_byte,HEX);
            if ((adresa%(maxdata/100)) == 0)          // zvetseni procent o 1
              {
                 lcd.GotoXY(54,5);
                 lcd.LcdNum((int)(adresa/(maxdata/100)),3,' ');
              }
          }


        delay200();
        Serial.println(231);     // desitkove 231 znamena konec prenosu  (231 uz se nepricita do CRC)
        delay100();


        print_crc(crc_ardu);

        Serial.println((unsigned long)crc_ardu % 256);  // pak se odesle jeste jeden bajt se zbytkem (MOD 256) kontrolniho souctu
        piip();
        delay(6000);


     }


   if (prichozi_byte == 87)   // znak W (Upload z PC do hodin)
     {
        lcd.LcdString("PC -> Hodiny");
        adresa = 0;
        while (adresa < 15500 && digitalRead(10) == HIGH)     // z PC do hodin se neprenasi FAT ani namerene udaje - posledni, co se prenasi, jsou alarmy
           {
             while (Serial.available() != 0)
               {
                  prichozi_byte = Serial.read();
                  crc_ardu = crc_ardu + prichozi_byte;

                  zapis_bajt(adresa,prichozi_byte);
                  adresa ++;

                  if(adresa%155 == 0)           //  po kazdych 155 bajtech se zvetsi pocitadlo  procent o 1
                    {
                      lcd.GotoXY(54,5);
                      lcd.LcdNum(((adresa/15499.0) * 100),3,' ');

                    }
               }

           }
        delay200();
        Serial.println(231);        // potvrzeni prijeti vsech 15499 bunek
        delay100();

        print_crc(crc_ardu);

        Serial.println(crc_ardu % 256);  // // pak se odesle jeste jeden bajt se zbytkem (MOD 256) kontrolniho souctu
        piip();
        delay(6000);
        jmp0();

     }


}




void menu()
{
  detachInterrupt(0); // zastavit aktualizaci zobrazeni casu na displeji
  lcd.LcdClear();
  inverzem = false;
  minipauza = millis();
  while (digitalRead(10) == LOW)   // dokud je stisknuty joystick nic se nedeje
    {
       delay200();       // osetreni zakmitani kontaktu
    }


  // zobrazeni menu
  lcd.GotoXY(6, 0);
  read_eeprom_txt(950, 3); // Menu: Jas
  lcd.GotoXY(6, 1);
  read_eeprom_txt(953, 7); // Menu: Zaznamy
  lcd.GotoXY(6, 2);
  read_eeprom_txt(960, 6); // Menu: Alarmy
  lcd.GotoXY(6, 3);
  read_eeprom_txt(966, 9); // Menu: Prenos PC
  lcd.GotoXY(6, 4);
  read_eeprom_txt(975, 3); // Menu: Cas
  lcd.GotoXY(6, 5);
  read_eeprom_txt(978, 9); // Menu: Nastaveni

  indexmenu = 0;   // indexmenu = 0 (prednastavena polozka JAS)
  sipecka(0);


  // dokud neni stisknuto tlacitko v joysticku, provadi sistovani po polozkach nahoru a dolu
  while ((digitalRead(10) == HIGH) && analogRead(A1) > 100)
    {
      joystick_y = analogRead(A0);

      if (minipauza + 120000 < millis()) jmp0();

      if (joystick_y < 100)
        {
          minipauza = millis();
          indexmenu --;
          if ( indexmenu < 0 ) {indexmenu = 0;}
          sipecka(indexmenu);
          delay500();
        }

      if (joystick_y > 800)
        {
          minipauza = millis();
          indexmenu ++;
          if ( indexmenu > 5 ) {indexmenu = 5;}
          sipecka(indexmenu);
          delay500();
        }
      delay(20);
    }

  // vystupem z liszovani je po stisku tlacitka promenna "indexmenu"

  lcd.LcdClear();
  delay200();

  lcdgoto00();   // smazani displeje a rozeskok na pozadovanou akci
  switch (indexmenu)
    {
      case 0:
        read_eeprom_txt(978, 9);    // Nastaveni: Jas
        lcd.LcdString(": ");
        read_eeprom_txt(950, 3);
        zmenajasu();
        break;

      case 1:
        ukaz_zaznamy();
        break;


      case 2:   // Alarmy
        if (ala_count > 0) listuj_alarmama(najdi_dalsi_alarm());
        else               listuj_alarmama(1);  // prvni uzivatelsky alarm
        break;

      case 3:             // komunikace mezi PC a hodinama (smer ovlada PC)
        komunikace(0);
        break;



      case 4:
        read_eeprom_txt(978, 9);    // "Nastaveni: Cas"
        kod58();
        read_eeprom_txt(975, 3);
        zmenacasu();
        break;


      default:
        menu_nastaveni1();
        break;


    }

   do
     {
       joystick_x = analogRead(A1);
       joystick_y = analogRead(A0);
     }
    while ((joystick_x > 800) || (joystick_x < 100) || (joystick_y > 800) || (joystick_y < 100));


  // po vykonanani zvolene akce z menu se displej smaze a spusti preruseni.
  // pokud je nastaveno aktivnbi mereni a ukladani hodnot, zalozi se novy FAT zaznam
  // Tim je rogram pripraven na navrat z menu do hlani smycky.
  delay500();
  lcd.LcdClear();
  novyFATzaznam();   // Novy FAT zaznam se zaklada pri vystupu z menu jen v pripade, ze je aktivni alespon jeden THUS flag

  cyklus_periody_save = perioda_save;

  attachInterrupt(0, preruseni, FALLING);

}




void aktivuj_budik(byte n)
{
  detachInterrupt(0);
  lcd.LcdClear();
  ukaz_alarmy(n,false);
  minipauza = millis();
  unsigned long startalarmu = minipauza;

  do
    {
      joystick_y = analogRead(A0);
      joystick_x = analogRead(A1);


      if ((millis() > minipauza + 1000 ) && bzuk_flag[1] == 1)
      {
         minipauza = millis();
         tone(A2, 800, 200);
      }

    }
  while ((joystick_x < 800) &&  (joystick_y < 800) && (joystick_x > 100) &&  (joystick_y > 100) && (startalarmu + 10000 > millis()));



  while (digitalRead(10) == HIGH)
    {
      zapis_bajt((64*(n-1))+9105,127);  // kdyz je alarm odklepnuty, tak se automaticky prepne na dalsi rok, ale deaktivuje se
      delay100();
    }


  lcd.LcdClear();



  attachInterrupt(0, preruseni, FALLING);
}






void setup()
{
  byte cisloFAT;
  unsigned int kombaud;


  ram_init();  // inicializace komunikace s RAM pameti



  // inicializuje diplej s nastavenim VOPu, TEMPu a BIASu
  lcd.LcdInitialise( precti_bajt(15),precti_bajt(16) , precti_bajt(17));


  // smaze displej
  lcd.LcdClear();



  rtc.init();  // vystup do 1Hz a spusteni generatoru



  byte rychkom = precti_bajt(14);

  switch (rychkom)
    {
      case 1:
        kombaud = 38400;
        break;
      case 2:
        kombaud = 57600;
        break;
      default:                  // = case 0
        kombaud = 9600;
    }



  Serial.begin(kombaud);  // start serial for output



  TCCR1B = TCCR1B & 0b11111000 | 0x02;    // nastaveni frekvence podsvetu na 3kHz (pri defaultnich 500Hz je obcas videt blikani)


  jas = precti_bajt(3); // precte uroven jasu z externi RAM
  lcd.podsvet(jas);


  napetimax1 = precti_bajt(18); // maximalni rozsah prvniho mereneho napeti (0 - 25,5V)
  napetimax2 = precti_bajt(19); // maximalni rozsah druheho mereneho napeti (0 - 25,5V)

  pinMode(A2, OUTPUT);         // piskak na "0"
  digitalWrite(A2, LOW);


  pinMode(A3, INPUT);         // analogovy vstup A3  (napeti 1)
  digitalWrite(A3, LOW);      // bez Pull-Upu

  pinMode(A7, INPUT);         // analogovy vstup A7  (napeti 2)
  digitalWrite(A7, LOW);      // bez Pull-Upu

  pinMode(A6, INPUT);         // analogovy vstup A6  (baterie)
  digitalWrite(A6, LOW);      // bez Pull-Upu



  durat_start = 0;


  inter_flag = false;


  ala_count = precti_bajt(7);
  ala_max   = precti_bajt(8);

  byte pomprom = precti_bajt(6);
  // nastaveni pipani podle hodnoty v SRAM
  bzuk_flag[0] = ((pomprom & 0b00001000) >> 3);  // H-odinove
  bzuk_flag[1] = ((pomprom & 0b00000100) >> 2);  // A-larmove
  bzuk_flag[2] = ((pomprom & 0b00000010) >> 1);  // J-oystickove
  bzuk_flag[3] =   pomprom & 0b00000001;         // S-ejfovaci


  rotace_podtextu = true;
  rotace_citac = 0;


  zapis_bajt(11,0);    // po resetu se rusi THUS flag (po zapnuti se neukladaji zadne hodnoty)
  menu_1_cisla(false);

  cisloFAT = precti_bajt(13);

  mem_proc  = (((-1*(precti_bajt(4)))+255)/1.83);   // procenta volneho mista

  if (cisloFAT > 200)
    {
      mem_proc  = 233;
    }
  else
    {
      adresaFAT = 15484 + (cisloFAT * 16); // adresa zacatku aktualni FAT  hlavicky
    }

  zaz_count=(precti_bajt(1)<<8) + precti_bajt(2);

  if (ala_count>0) index_dalsiho_alarmu = najdi_dalsi_alarm();



  if ( analogRead(A0) < 100)
    {
      menu_nastaveni2();
    }




  cyklus_periody_save = perioda_save;

  attachInterrupt(0, preruseni, FALLING);
}


void loop()
{


  if (inter_flag == true)          // zobrazeni na displeji se provadi 1x za sekundu (frekvence preruseni je 1Hz)
    {


      zobrazcas();                 // po kazdem preruseni se aktualizuje displej




      rtc_do_datcas();  // aktualni cas v RTC se prepise do 7 globalnich desitkovych promennych (rokdec, mesdec, dendec ..... mscdec)

      if(hoddec == 0 && mindec==0 && sekdec==0 && dendec==1 && mesdec==1)  // na zacatku roku (pokud je program spusteny)
        {
           najdi_dalsi_alarm();           // prepocitej alarmy, ktere byly v nasledujicim roce na aktualni rok *
        }
                      // * priklad: je Silvestr roku 2014 a dalsi alarm je nastaven na srpen (automaticky se tedy pocita s tim, ze je to srpen 2015)
                      // aktualni cas je 1231230000 a srpnovy alarm je 2005234500 (srpen + 1231235959).
                      // pri prechodu pres pulnoc se aktualni cas nastavi na 0101000000, ale zaroven se srpnovy alarm musi prepoocitat na 0805234500

      aktcas = (mesdec * 100000000UL) +  (dendec * 1000000UL) +  (hoddec * 10000UL) + (mindec * 100UL) + sekdec;


      if (aktcas == cas_dalsiho_alarmu)
        {
          aktivuj_budik(dalsi_alarm);
          najdi_dalsi_alarm();
          lcd.LcdClear();
        }


      lcd.ramecek(ala_count , zaz_count , mem_proc, inverzem); // a obnovi se i inforamecek






      inter_flag = false;           //   a znacka, ze doslo k preruseni, se vynuluje
    }

  if (digitalRead(10) == LOW)  menu();   // po stisku joysticku se vleze do menu

  joystick_y = analogRead(A0);
  joystick_x = analogRead(A1);

  if ((joystick_x > 800) || (joystick_x < 100) || (joystick_y > 800) || (joystick_y < 100) || Serial.available()>0)  // joystick byl nekam vychylen
    {
      if (uvolneno == true)               // pokud byl predti uvolnen, dojde k zaznamu casu
        {
          durat_start = millis();
          if (Serial.available()>0) smer_j = Serial.read(); // pri zaznamu casu pres PC se jako smer uklada prijaty znak

          if (joystick_x > 800) smer_j = 76;    // znak L (vlevo)
          if (joystick_x < 100) smer_j = 80;    // znak P (vpravo)
          if (joystick_y > 800) smer_j = 68;    // znak D (dolu)
          if (joystick_y < 100) smer_j = 78;    // znak N (nahoru)

          if (smer_j == 81 || smer_j == 87 )    // "Q", "W" - prenos dat pres komunikaci bez nutnosti lezt do menu (PC urcuje smer komunikace - Q /W)
            {
              detachInterrupt(0);
              lcd.LcdClear();
              lcdgoto00();
              komunikace(smer_j);
              lcd.LcdClear();
              novyFATzaznam();
              attachInterrupt(0, preruseni, FALLING);
            }
          else
            {


              zaz_count ++;
              if (zaz_count > 999) zaz_count = 999;   // pri prekroceni 999 zaznamu se prepisuje jen 999. zaznam. Stare zaznamy zustavaji v pameti
              zaznamcasu(smer_j);
              uvolneno = false;
              if (smer_j == 83 )   // "S" = znacka pro porovnani dvou casu
                {
                  detachInterrupt(0);     // odeslani zaznamenaneho casu po seriove lince
                  Serial.println(hoddec,HEX);
                  Serial.println(mindec,HEX);
                  Serial.println(sekdec,HEX);
                  Serial.println(mscdec,HEX);
                  Serial.println(123);
                  attachInterrupt(0, preruseni, FALLING);

                }

            }
        }
    }

  if ((joystick_x < 700) && (joystick_x > 200) && (joystick_y < 700) && (joystick_y > 200)) // joystick byl vyrovnan
    {
     if (uvolneno == false && zaz_count > 0)    // pokud byl predtim vychylen, zaznamena se delka casu vychyleni
       {
         durat_j=(millis() - durat_start) / 100 ;  // (durat_j je v desetinach sekundy)
         if (durat_j > 255) durat_j=255;           // pro prekroceni 25,5 se ulozi hodnota 25,5 sekundy (cislo 255)
         adresa = (zaz_count * 9) + 11;            // nastaveni adresy na bunku pro zapis casoveho trvani vychyleni joysticku
         zapis_bajt (adresa + 8, durat_j);         // zapis delky casu vychyleni do externi SRAM

         if ((durat_j > 20) && (bzuk_flag[2]==1)) tone(A2, 400, 100);     // pro durat delsi, nez 2sek oznam hlubsim tonem uvolneni joysticku

       }


      lcdgoto05();                       // pokud pri narovnani joysticku ubehlo alespon 5 sekund od vycyleni, cas pod inforameckem se smaze
      if (minipauza + 5000 < millis())
        {

          rotace_podtextu = true;

        }
      uvolneno = true;                    // pri uvolneni se nastvi promenna uvolneno na true okamzite
    }


}




