#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#define byte uint8_t
#define send( X ) write( static_cast<uint8_t>(X) )
#define receive( X ) read( X )
#else
#include "WProgram.h"
#endif

//#include <Wire.h>
#include "rtc_m41t81.h"


//prevod z desitkoveho cisla na dva pulbajty (desitky a jednotky)
byte rtc_m41t81::DecToBCD(byte val)
{
  return ( (val/10*16) + (val%10) );
}


// prevod ze dvou pulbajtu (desitky a jednotky) na desitkove cislo
byte rtc_m41t81::BCDToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
}


void rtc_m41t81::init(void)
{

  Wire.begin();

  Wire.beginTransmission(0x68);	   // Start I2C komunikace
  Wire.write(0x0C); 	             // v registru c. 0x0C
  Wire.write(0x00);                // HALT bit do 0
  Wire.endTransmission();


  Wire.beginTransmission(0x68);	   // Start I2C komunikace
  Wire.write(0x0A); 	             // v registru c. 0x0A
  Wire.write(0x40);                // SQWE enable
  Wire.endTransmission();


  Wire.beginTransmission(0x68);	   // Start I2C komunikace
  Wire.write(0x13); 	             // v registru c. 0x13
  Wire.write(0xF0);                // delic frekvence na 1Hz
  Wire.endTransmission();



}



void rtc_m41t81::setTime(byte hod, byte minu, byte sek, byte den, byte mes, byte rok )
{

  Wire.beginTransmission(0x68);	   // Start I2C komunikace
  Wire.write(0x00);   	           // nastaveni ukazatele zapisu na nulty registr


  // prevod z desitkové soustavy (0 až 59) na binarni desitky a jednotky
  hod = DecToBCD(hod);
  minu = DecToBCD(minu);
  sek = DecToBCD(sek);

  den = DecToBCD(den);
  mes = DecToBCD(mes);
  rok = DecToBCD(rok);



  Wire.write(0); 		    // [0x00] nastaveni setin sekund na 0 (nastavovani se musi provadet v celou sekundu)
  Wire.write(sek); 	    // [0x01] set seconds + vynulovat stopbit
  Wire.write(minu);	    // [0x02] set minutes
  Wire.write(hod+192);  // [0x03] set hour + 21. stoleti + enable stoleti (0b11000000)

  Wire.write(1);	      // [0x04] den v tydnu se nepouziva, tak se nastavi na 1 (pondeli)
  Wire.write(den);	    // [0x05] dny v mesici
  Wire.write(mes);	    // [0x06] mesice
  Wire.write(rok);	    // [0x07] roky (posledni dvojcisli xx) predpoklada se rok 20xx


  Wire.endTransmission();


}




void rtc_m41t81::setKalib( byte kalibrace )
{

  Wire.beginTransmission(0x68);	   // Start I2C komunikace
  Wire.write(0x08);   	           // nastaveni ukazatele zapisu na registr zacatku alarmu


  Wire.write(kalibrace & 0b00111111);	    // [0x08] dva nejvyssi bity jsou v "0" (OUT a FT) zbytek je sign kalibrace

  Wire.endTransmission();

}



void rtc_m41t81::getTime()
{

	/* set the start byte (nulty registr (setiny sekundy)) */
	Wire.beginTransmission(0x68);
 	Wire.write(0x00);
 	Wire.endTransmission();



 	Wire.requestFrom(0x68, 9);        //bude se cist blok 9 bajtu

  msek = (Wire.read() & 0xFF);    // [0x00] - setiny sekund
 	sek  = (Wire.read() & 0x7F);    // [0x01] - sekundy
	minu  = (Wire.read() & 0x7F);    // [0x02] - minuty
	hod  = (Wire.read() & 0x3F);    // [0x03] - hodiny

	den  = (Wire.read() & 0x07);    // [0x04] - den v tydnu (nepouziva se)
	den  = (Wire.read() & 0x3F);    // [0x05] - dny v mesici
	mes  = (Wire.read() & 0x1F);    // [0x06] - mesice
	rok  = (Wire.read() & 0xFF);    // [0x07] - posledni dvojcisli roku

	kalibrace  = (Wire.read() & 0x3F);    // [0x08] - kalibrace krystalu


  msek = BCDToDec(msek);
  sek  = BCDToDec(sek);
  minu  = BCDToDec(minu);
  hod  = BCDToDec(hod);

  den  = BCDToDec(den);
  mes  = BCDToDec(mes);
  rok  = BCDToDec(rok);


}



char *rtc_m41t81::formatTime(byte styl)
{
	getTime();
  switch (styl)
   {
    case 1:  // vsechny informace zhustene v jednom bloku
    	strOut[0] = '0' + (hod / 10);
     	strOut[1] = '0' + (hod % 10);
    	strOut[2] = '0' + (minu / 10);
     	strOut[3] = '0' + (minu % 10);
    	strOut[4] = '0' + (sek / 10);
     	strOut[5] = '0' + (sek % 10);
    	strOut[6] = '0' + (msek / 10);
     	strOut[7] = '0' + (msek % 10);

    	strOut[8] = '0' + (den / 10);
     	strOut[9] = '0' + (den % 10);
    	strOut[10] = '0' + (mes / 10);
     	strOut[11] = '0' + (mes % 10);
    	strOut[12] = '0' + (rok / 10);
     	strOut[13] = '0' + (rok % 10);

    	strOut[14] = '0' + (kalibrace / 10);
     	strOut[15] = '0' + (kalibrace % 10);
     	strOut[16] = '\0';
      break;

    case 2:      // orezany, ale citelny text
    	strOut[0] = '0' + (hod / 10);
     	strOut[1] = '0' + (hod % 10);
     	strOut[2] = ':';
    	strOut[3] = '0' + (minu / 10);
     	strOut[4] = '0' + (minu % 10);
     	strOut[5] = ':';
    	strOut[6] = '0' + (sek / 10);
     	strOut[7] = '0' + (sek % 10);
     	strOut[8] = '\0';
     	strOut[9] = '0' + (mes / 10);
     	strOut[10] = '0' + (mes % 10);
     	strOut[11] = '0' + (den / 10);
     	strOut[12] = '0' + (den % 10);

      break;
    }

 	return strOut;
}



